<?php

namespace Drupal\nginx_proxy_manager_connector\Services;

use GuzzleHttp\ClientInterface;
use Drupal\Component\Serialization\Json;
use Drupal\Core\Config\ConfigFactoryInterface;

/**
 * Service to integrate Nginx Proxy Manager.
 */
class NginxProxyManagerService {

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The HTTP client to fetch the feed data with.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * Constructor for NginxProxyManagerService.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   A config factory for retrieving required config objects.
   * @param \GuzzleHttp\ClientInterface $httpClient
   *   A Guzzle client object.
   */
  public function __construct(ConfigFactoryInterface $configFactory, ClientInterface $httpClient) {
    $this->httpClient = $httpClient;
    $this->config = $configFactory->get('nginx_proxy_manager_connector.nginxproxysettings');
    $this->api_calls_base_url = $this->config->get('api_base_url');
    $this->master_domain = $_ENV['MASTER_DOMAIN'];
  }

  /**
   * Create Proxy Host in NginxProxyManager.
   *
   * @param string $custom_domain
   * @param string $custom_subdomain
   * @param string $forward_host
   * @param string $port
   * @param string $letsencrypt_email
   *
   * @return array $response
   */
  public function create_proxy_host($custom_domain, $custom_subdomain, $forward_host = NULL, $port = NULL, $letsencrypt_email = NULL) {

    // Initiate API calls configuration.
    if (!$this->api_call_init()) {
      return ['failure' => 'Proxy host not created.'];
    }

    // Set Defaults.
    if ($forward_host == NULL) {
      $forward_host = $_ENV['NGINX_PROXY_MANAGER_DEFAULT_FORWARD_HOST'];
    }
    if ($port == NULL) {
      $port = $this->config->get('default_forward_host_port');
    }
    if ($letsencrypt_email == NULL) {
      $letsencrypt_email = $this->config->get('letsencrypt_email');
    }

    $error_message = '';

    // Create domain names string.
    if ($custom_domain !== '') {
      $non_www_domain_name = preg_replace('/^www\./', '', $custom_domain);
      $www_domain_name = 'www.' . $non_www_domain_name;
      $custom_domain_domain_names = '"' . $non_www_domain_name . '", "' . $www_domain_name . '"';

      $proxy_host_id = $this->get_proxy_host_id($non_www_domain_name);

      if ($proxy_host_id == 0) {
        // Create Proxy Host for custom domain.
        $api_response_custom_domain = Json::decode($this->api_call_create_proxy_host_without_certificate($custom_domain_domain_names, $forward_host, $port));
      }
      else {
        $api_response_custom_domain = Json::decode($this->api_call_enable_proxy_host($proxy_host_id));
      }
      // If error, send appropriate failure response.
      // Show subdomain error, if both throw error.
      if (isset($api_response_custom_domain['error'])) {
        $error_message = $error_message . $api_response_custom_domain['error']['message'];
        \Drupal::logger('nginx_proxy_manager_connector')->error("Nginx Proxy Manager Error, Code:" . $api_response_custom_domain['error']['code'] . ", Message:" . $api_response_custom_domain['error']['message']);
      }
    }

    if ($this->get_proxy_host_id($custom_subdomain) == 0) {

      $custom_subdomain_domain_names = '"' . $custom_subdomain . '"';

      // Get certificate for custom subdomain and create if needed.
      $this->api_call_create_certificate($custom_subdomain, $letsencrypt_email);

      $certificate_id = $this->get_certificate_id($custom_subdomain);

      // Create Proxy Host for custom subdomain.
      $api_response_custom_subdomain = Json::decode($this->api_call_create_proxy_host($custom_subdomain_domain_names, $forward_host, $certificate_id, $port));
      // If error, send appropriate failure response.
      // Show subdomain error, if both throw error.
      if (isset($api_response_custom_subdomain['error'])) {
        $error_message = $error_message . $api_response_custom_subdomain['error']['message'];
        \Drupal::logger('nginx_proxy_manager_connector')->error("Nginx Proxy Manager Error, Code:" . $api_response_custom_subdomain['error']['code'] . ", Message:" . $api_response_custom_subdomain['error']['message']);
      }
    }

    if ($error_message !== '') {
      $response = ['failure' => $error_message];
    }
    else {
      $response = ['success' => TRUE];
    }

    return $response;
  }

  /**
   * Deploy Proxy Host in NginxProxyManager.
   *
   * @param string $custom_domain
   * @param int $proxy_host_id
   *
   * @return bool
   */
  public function deploy_proxy_host($custom_domain, $proxy_host_id = 0) {

    // Initiate API calls configuration.
    if (!$this->api_call_init()) {
      return ['failure' => 'Proxy host not enabled.'];
    }

    // If Proxy Host Id is not provided find it by custom_domain.
    if ($proxy_host_id == 0) {
      $proxy_host_id = $this->get_proxy_host_id($custom_domain);
    }
    $response = Json::decode($this->api_call_enable_proxy_host($proxy_host_id));
    if (isset($response['error'])) {
      \Drupal::logger('nginx_proxy_manager_connector')->error("Nginx Proxy Manager Error, Code:" . $response['error']['code'] . ", Message:" . $response['error']['message']);
    }
    return $response;
  }

  /**
   * Disable Proxy Host in NginxProxyManager.
   *
   * @param string $custom_domain
   * @param int $proxy_host_id
   *
   * @return bool
   */
  public function disable_proxy_host($custom_domain, $proxy_host_id = 0) {
    // Initiate API calls configuration.
    if (!$this->api_call_init()) {
      return ['failure' => 'Proxy host not disabled.'];
    }

    // If Proxy Host Id is not provided find it by custom_domain.
    if ($proxy_host_id == 0) {
      $proxy_host_id = $this->get_proxy_host_id($custom_domain);
    }
    $response = Json::decode($this->api_call_disable_proxy_host($proxy_host_id));
    if (isset($response['error'])) {
      \Drupal::logger('nginx_proxy_manager_connector')->error("Nginx Proxy Manager Error, Code:" . $response['error']['code'] . ", Message:" . $response['error']['message']);
    }
    return $response;
  }

  /**
   * Update Proxy Host in NginxProxyManager.
   *
   * @param string $old_custom_domain
   * @param string $new_custom_domain
   * @param int $proxy_host_id
   * @param int $enable_ssl
   *
   * @return bool
   */
  public function update_proxy_host($old_custom_domain, $new_custom_domain, $proxy_host_id = 0, $enable_ssl = 0) {

    // Initiate API calls configuration.
    if (!$this->api_call_init()) {
      return ['failure' => 'Proxy host not updated.'];
    }

    // If Proxy Host Id is not provided find it by custom_domain.
    if ($old_custom_domain !== "" && $proxy_host_id == 0) {
      $proxy_host_id = $this->get_proxy_host_id($old_custom_domain);
    }

    // Set domain names.
    $non_www_domain_name = preg_replace('/^www\./', '', $new_custom_domain);
    $www_domain_name = 'www.' . $non_www_domain_name;
    $custom_domain_domain_names = '"' . $non_www_domain_name . '", "' . $www_domain_name . '"';

    if ($proxy_host_id == 0) {
      // Set Defaults.
      $config = \Drupal::config('nginx_proxy_manager_connector.nginxproxysettings');
      $forward_host = $_ENV['NGINX_PROXY_MANAGER_DEFAULT_FORWARD_HOST'];
      $port = $config->get('default_forward_host_port');
      // Create Proxy Host for custom domain.
      $response = Json::decode($this->api_call_create_proxy_host_without_certificate($custom_domain_domain_names, $forward_host, $port));
    }
    else {
      $response = Json::decode($this->api_call_update_proxy_host($custom_domain_domain_names, $proxy_host_id));
    }

    if ($enable_ssl) {
      $cert_response = $this->update_proxy_host_certificate($new_custom_domain);
      if (isset($cert_response['error'])) {
        \Drupal::logger('nginx_proxy_manager_connector')->error("Nginx Proxy Manager Error, Code:" . $response['error']['code'] . ", Message:" . $response['error']['message']);
      }
    }

    if (isset($response['error'])) {
      \Drupal::logger('nginx_proxy_manager_connector')->error("Nginx Proxy Manager Error, Code:" . $response['error']['code'] . ", Message:" . $response['error']['message']);
    }
    return $response;
  }

  /**
   * Update Proxy Host in NginxProxyManager.
   *
   * @param string $custom_domain
   * @param string $action
   * @param string $letsencrypt_email
   * @param int $proxy_host_id
   *
   * @return bool
   */
  public function update_proxy_host_certificate($custom_domain, $action = "add", $letsencrypt_email = NULL, $proxy_host_id = 0) {

    // Initiate API calls configuration.
    if (!$this->api_call_init()) {
      return ['failure' => 'Proxy host not updated.'];
    }
    if ($custom_domain == '') {
      return ['failure' => 'Empty custom domain'];
    }
    // Set De$config = \Drupal::config('nginx_proxy_manager_connector.nginxproxysettings');faults.
    $config = \Drupal::config('nginx_proxy_manager_connector.nginxproxysettings');
    if ($letsencrypt_email == NULL) {
      $letsencrypt_email = $config->get('letsencrypt_email');
    }
    if ($action == "add") {

      // Check if certificate exists.
      $certificate_id_from_list = $this->get_certificate_id($custom_domain);
      // Create if it doesn't exist.
      if ($certificate_id_from_list == 0) {
        // Create certificate.
        $this->api_call_create_certificate($custom_domain, $letsencrypt_email);
        $certificate_id_from_list = $this->get_certificate_id($custom_domain);

        if ($certificate_id_from_list == 0) {
          return ["failure" => "Certificate not generated"];
        }
      }
      $certificate_id = $certificate_id_from_list;
    }
    elseif ($action = "remove") {
      $certificate_id = 0;
    }

    // If Proxy Host Id is not provided find it by custom_domain.
    if ($proxy_host_id == 0) {
      $proxy_host_id = $this->get_proxy_host_id($custom_domain);
    }

    $response = Json::decode($this->api_call_update_proxy_host_certificate($certificate_id, $proxy_host_id));
    if (isset($response['error'])) {
      \Drupal::logger('nginx_proxy_manager_connector')->error("Nginx Proxy Manager Error, Code:" . $response['error']['code'] . ", Message:" . $response['error']['message']);
    }
    return $response;
  }

  /**
   * Get Proxy Host Id using domain name.
   *
   * @param string $custom_domain
   *
   * @return int
   */
  public function get_proxy_host_id($custom_domain) {
    // Initiate API calls configuration.
    if (!$this->api_call_init()) {
      return ['failure' => 'Proxy host not created.'];
    }

    // Get list of proxy hosts.
    $proxy_hosts_list = Json::decode($this->api_call_get_proxy_hosts());

    foreach ($proxy_hosts_list as $proxy_host) {
      if (in_array($custom_domain, $proxy_host['domain_names'])) {
        return $proxy_host['id'];
      }
    }
    return 0;
  }

  /**
   * Get Proxy Host Id using domain name.
   *
   * @param string $custom_domain
   *
   * @return int
   */
  public function get_proxy_host_certificate($custom_domain) {
    // Initiate API calls configuration.
    if (!$this->api_call_init()) {
      return ['failure' => 'Proxy host not created.'];
    }

    // Get list of proxy hosts.
    $proxy_hosts_list = Json::decode($this->api_call_get_proxy_hosts());

    foreach ($proxy_hosts_list as $proxy_host) {
      if (in_array($custom_domain, $proxy_host['domain_names'])) {
        return $proxy_host['certificate_id'];
      }
    }
    return 0;
  }

  /**
   * Get Certificate Id using domain name.
   *
   * @param string $custom_domain
   *
   * @return int
   */
  public function get_certificate_id($custom_domain) {
    // Initiate API calls configuration.
    if (!$this->api_call_init()) {
      return ['failure' => 'Proxy host not created.'];
    }

    // Get list of proxy hosts.
    $certificates_list = Json::decode($this->api_call_get_certificates());

    foreach ($certificates_list as $certificate) {
      if (in_array($custom_domain, $certificate['domain_names'])) {
        return $certificate['id'];
      }
    }

    return 0;
  }

  // ----------------------- API Calls ------------------------------

  /**
   * Check Nginx Proxy Manager status and generate token.
   *
   * @return array
   */
  public function api_call_init() {

    // Check if Nginx Proxy Manager is online.
    if (Json::decode($this->api_call_health_status())['status'] == 'OK') {

      // Check if user credentials are configured.
      if ($_ENV['NGINX_PROXY_MANAGER_USER_EMAIL'] !== NULL && $_ENV['NGINX_PROXY_MANAGER_PASSWORD'] !== NULL) {

        // Generate token.
        if (isset(Json::decode($this->api_call_generate_token())['token'])) {
          \Drupal::state()->set('api_calls_token', Json::decode($this->api_call_generate_token())['token']);
          return TRUE;
        }
        else {
          \Drupal::logger('nginx_proxy_manager_connector')->error("Please provide valid user credentials for Nginx Proxy Manager");
          return FALSE;
        }
      }
      \Drupal::logger('nginx_proxy_manager_connector')->error("Please configure user credentials for Nginx Proxy Manager");
      return FALSE;
    }

    \Drupal::logger('nginx_proxy_manager_connector')->error("Nginx Proxy Manager is offline");
    return FALSE;
  }

  /**
   * API Call for checking Nginx Proxy Manager status.
   *
   * @return \Drupal\Component\Serialization\Json
   */
  public function api_call_health_status() {

    // Make request and return response.
    return $this->httpClient->request('GET', $this->api_calls_base_url)->getBody()->getContents();
  }

  /**
   * API Call for generating token for authentication.
   *
   * @return \Drupal\Component\Serialization\Json $response_json
   */
  public function api_call_generate_token() {

    // Set data.
    $data = [
      'identity' => $_ENV['NGINX_PROXY_MANAGER_USER_EMAIL'],
      'scope' => 'user',
      'secret' => $_ENV['NGINX_PROXY_MANAGER_PASSWORD'],
    ];

    // Make request.
    $response_json = $this->httpClient->request(
          'POST',
          $this->api_calls_base_url . 'tokens',
          [
            'body' => json_encode($data, JSON_FORCE_OBJECT),
            'headers' => [
              'Accept' => 'application/json',
              'Content-Type' => 'application/json',
            ],
          ]
      )->getBody()->getContents();

    // Return response.
    return $response_json;
  }

  /**
   * API Call.
   *
   * @param string $domain_names
   * @param string $forward_host
   * @param int $certificate_id
   * @param string $port
   *
   * @return \Drupal\Component\Serialization\Json $json
   */
  public function api_call_create_proxy_host($domain_names, $forward_host, $certificate_id, $port) {

    // Set data.
    $data = [
      'domain_names' => "",
      'forward_scheme' => "https",
      'forward_host' => $forward_host,
      'forward_port' => $port,
      'certificate_id' => $certificate_id,
      'ssl_forced' => TRUE,
      'hsts_enabled' => TRUE,
      'hsts_subdomains' => FALSE,
      'http2_support' => FALSE,
      'block_exploits' => TRUE,
      'caching_enabled' => FALSE,
      'allow_websocket_upgrade' => FALSE,
      'access_list_id' => '0',
      'advanced_config' => "",
      'enabled' => TRUE,
    ];

    $data = str_replace('"domain_names":""', '"domain_names": [' . $domain_names . ']', json_encode($data, JSON_FORCE_OBJECT));

    // Make request.
    $response_json = $this->httpClient->request(
          'POST',
          $this->api_calls_base_url . 'nginx/proxy-hosts',
          [
            'body' => $data,
            'headers' => [
              'Authorization' => 'Bearer ' . \Drupal::state()->get('api_calls_token'),
              'Accept' => 'application/json',
              'X-API-Version' => 'next',
              'Content-Type' => 'application/json',
            ],
          ]
      )->getBody()->getContents();

    // Return response.
    return $response_json;
  }

  /**
   * API Call.
   *
   * @param string $domain_names
   * @param string $forward_host
   * @param string $port
   *
   * @return \Drupal\Component\Serialization\Json $json
   */
  public function api_call_create_proxy_host_without_certificate($domain_names, $forward_host, $port) {

    // Set data.
    $data = [
      'domain_names' => "",
      'forward_scheme' => "https",
      'forward_host' => $forward_host,
      'forward_port' => $port,
      'certificate_id' => '0',
      'ssl_forced' => FALSE,
      'hsts_enabled' => FALSE,
      'hsts_subdomains' => FALSE,
      'http2_support' => FALSE,
      'block_exploits' => TRUE,
      'caching_enabled' => FALSE,
      'allow_websocket_upgrade' => FALSE,
      'access_list_id' => '0',
      'advanced_config' => "",
      'enabled' => TRUE,
    ];

    $data = str_replace('"domain_names":""', '"domain_names": [' . $domain_names . ']', json_encode($data, JSON_FORCE_OBJECT));

    // Make request.
    $response_json = $this->httpClient->request(
          'POST',
          $this->api_calls_base_url . 'nginx/proxy-hosts',
          [
            'body' => $data,
            'headers' => [
              'Authorization' => 'Bearer ' . \Drupal::state()->get('api_calls_token'),
              'Accept' => 'application/json',
              'X-API-Version' => 'next',
              'Content-Type' => 'application/json',
            ],
          ]
      )->getBody()->getContents();

    // Return response.
    return $response_json;
  }

  /**
   * API Call.
   *
   * @return \Drupal\Component\Serialization\Json $json
   */
  public function api_call_get_proxy_hosts() {

    // Make request.
    $response_json = $this->httpClient->request(
          'GET',
          $this->api_calls_base_url . 'nginx/proxy-hosts',
          [
            'headers' => [
              'Authorization' => 'Bearer ' . \Drupal::state()->get('api_calls_token'),
              'X-API-Version' => 'next',
              'Accept' => 'application/json',
              'Content-Type' => 'application/json',
            ],
          ]
      )->getBody()->getContents();

    // Return response.
    return $response_json;
  }

  /**
   * API Call.
   *
   * @param int $proxy_host_id
   *
   * @return bool
   */
  public function api_call_enable_proxy_host($proxy_host_id) {

    // Make request.
    $response_json = $this->httpClient->request(
          'POST',
          $this->api_calls_base_url . 'nginx/proxy-hosts/' . $proxy_host_id . '/enable',
          [
            'headers' => [
              'Authorization' => 'Bearer ' . \Drupal::state()->get('api_calls_token'),
              'X-API-Version' => 'next',
              'Accept' => 'application/json',
              'Content-Type' => 'application/json',
            ],
          ]
      )->getBody()->getContents();

    // Return response.
    return $response_json;
  }

  /**
   * API Call.
   *
   * @param int $proxy_host_id
   *
   * @return bool
   */
  public function api_call_disable_proxy_host($proxy_host_id) {
    // Make request.
    $response_json = $this->httpClient->request(
          'POST',
          $this->api_calls_base_url . 'nginx/proxy-hosts/' . $proxy_host_id . '/disable',
          [
            'headers' => [
              'Authorization' => 'Bearer ' . \Drupal::state()->get('api_calls_token'),
              'X-API-Version' => 'next',
              'Accept' => 'application/json',
              'Content-Type' => 'application/json',
            ],
          ]
      )->getBody()->getContents();

    // Return response.
    return $response_json;
  }

  /**
   * API Call.
   *
   * @param string $domain_names
   * @param int $proxy_host_id
   *
   * @return \Drupal\Component\Serialization\Json $json
   */
  public function api_call_update_proxy_host($domain_names, $proxy_host_id) {
    // Set data.
    $data = ["domain_names" => ""];
    $data = str_replace('"domain_names":""', '"domain_names": [' . $domain_names . ']', json_encode($data, JSON_FORCE_OBJECT));

    // Make request.
    $response_json = $this->httpClient->request(
          'PUT',
          $this->api_calls_base_url . 'nginx/proxy-hosts/' . $proxy_host_id,
          [
            'body' => $data,
            'headers' => [
              'Authorization' => 'Bearer ' . \Drupal::state()->get('api_calls_token'),
              'Accept' => 'application/json',
              'X-API-Version' => 'next',
              'Content-Type' => 'application/json',
            ],
          ]
      )->getBody()->getContents();

    return $response_json;
  }

  /**
   * API Call.
   *
   * @param int $certificate_id
   * @param int $proxy_host_id
   *
   * @return \Drupal\Component\Serialization\Json $json
   */
  public function api_call_update_proxy_host_certificate($certificate_id, $proxy_host_id) {

    // Set data.
    $data = [
      "certificate_id" => $certificate_id,
      'ssl_forced' => TRUE,
      'hsts_enabled' => TRUE,
    ];

    // Make request.
    $response_json = $this->httpClient->request(
          'PUT',
          $this->api_calls_base_url . 'nginx/proxy-hosts/' . $proxy_host_id,
          [
            'body' => json_encode($data, JSON_FORCE_OBJECT),
            'headers' => [
              'Authorization' => 'Bearer ' . \Drupal::state()->get('api_calls_token'),
              'Accept' => 'application/json',
              'X-API-Version' => 'next',
              'Content-Type' => 'application/json',
            ],
          ]
      )->getBody()->getContents();
    return $response_json;
  }

  /**
   * API Call.
   *
   * @param string $custom_domain
   * @param string $letsencrypt_email
   *
   * @return \Drupal\Component\Serialization\Json $json
   */
  public function api_call_create_certificate($custom_domain, $letsencrypt_email) {
    $non_www_domain_name = preg_replace('/^www\./', '', $custom_domain);
    $www_domain_name = 'www.' . $non_www_domain_name;

    $domain_names = '"' . $non_www_domain_name . '", "' . $www_domain_name . '"';

    // Set data.
    $data = [
      "domain_names" => "",
      "provider" => "letsencrypt",
      "nice_name" => "test",
      "meta" => [
        "letsencrypt_email" => $letsencrypt_email,
        "letsencrypt_agree" => TRUE,
        "dns_challenge" => FALSE,
      ],
    ];

    $data = str_replace('"domain_names":""', '"domain_names": [' . $domain_names . ']', json_encode($data, JSON_FORCE_OBJECT));

    // Make request.
    $response_json = $this->httpClient->request(
          'POST',
          $this->api_calls_base_url . 'nginx/certificates',
          [
            'body' => $data,
            'headers' => [
              'Authorization' => 'Bearer ' . \Drupal::state()->get('api_calls_token'),
              'Accept' => 'application/json',
              'X-API-Version' => 'next',
              'Content-Type' => 'application/json',
            ],
          ]
      )->getBody()->getContents();

    return $response_json;

  }

  /**
   * API Call for getting list og all certificates.
   *
   * @return \Drupal\Component\Serialization\Json $json
   */
  public function api_call_get_certificates() {
    // Make request.
    $response_json = $this->httpClient->request(
          'GET',
          $this->api_calls_base_url . 'nginx/certificates',
          [
            'headers' => [
              'Authorization' => 'Bearer ' . \Drupal::state()->get('api_calls_token'),
              'X-API-Version' => 'next',
              'Accept' => 'application/json',
              'Content-Type' => 'application/json',
            ],
          ]
      )->getBody()->getContents();

    // Return response.
    return $response_json;
  }

}
